from enum import Enum, auto

class RoFeaturesName(Enum):
    ABBR = 'Abbreviation'
    ADPT = 'AdpType'
    CASE = 'Case'
    DEF = 'Definite'
    DEG = 'Degree'
    FGN = 'Foreign'
    GEN = 'Gender'
    MOOD = 'Mood'
    NR = 'Number'
    NRP = 'Number[psor]'
    NRF = 'NumForm'
    NRT = 'NumType'
    PARTT = 'PartType'
    PER = 'Person'
    POL = 'Polarity'
    POSI = 'Position'
    POSS = 'Poss'
    PRONT = 'PronType'
    RFL = 'Reflex'
    STRN = 'Strength'
    TENSE = 'Tense'
    VAR = 'Variant'
    VBF = 'VerbForm'
     